import React, { useEffect, useRef, useState } from "react";
import MoveableHelper from "moveable-helper";
import Moveable from "react-moveable";
import parse from "html-react-parser";
import classNames from "classnames";
import InputField from "components/Common/InputField";
import { isValidArray, pxTomm } from "modules/utils";
import imagePlaceholder from "assets/images/imagePlaceholder.jpg";
import TextField from "./TextField";
import Loader from "components/Common/Loader";
import { useSelector } from "react-redux";

export const defaultMoveableProps = {
  draggable: true,
  rotatable: true,
  snappable: true,
  snapThreshold: 5,
  isDisplaySnapDigit: false,
  snapGap: true,
  snapDirections: {
    top: true,
    right: true,
    bottom: true,
    left: true,
    center: true,
    middle: true,
  },
  elementSnapDirections: {
    top: true,
    right: true,
    bottom: true,
    left: true,
    center: true,
    middle: true,
  },
  snapDigit: 5,
  zoom: 1,
  origin: false,
};

const MoveableElement = (props) => {
  const {
    activeId,
    blockHeight,
    blockWidth,
    initialized,
    isActive: isFieldActive,
    properties,
    selectedElement,
    setActiveId,
    setInitialized,
    setTargets,
    targets,
    updateElement,
  } = props;

  const {
    backgroundColor,
    borderRadius = 0,
    color,
    fontWeight,
    height,
    left,
    top,
    width,
    padding,
    zIndex,
    rotate,
    customCSS,
    strokeColor,
  } = properties || {};

  const [helper] = useState(() => new MoveableHelper());
  const [keepRatio, setKeepRatio] = useState(false);
  const [elementGuidelines, setElementGuidelines] = useState([]);
  const [tempActive, setTempActive] = useState(false);

  const templateRef = useRef(null);
  const moveable = useRef(null);

  const { isLoading = false } = useSelector((state) => ({
    isLoading: state.getBlock?.loading,
  }));

  const checkFieldActive = () => {
    if (activeId.length > 1 && isFieldActive) {
      return activeId[0] === properties.id;
    }
    return isFieldActive;
  };
  const isActive = checkFieldActive();

  const detectExpansionFN = () => {
    switch (properties?.expansionAnchor) {
      case "tc":
        return "n";
      case "tl":
        return "nw";
      case "tr":
        return "ne";
      case "ml":
        return "w";
      case "mc":
        return "n";
      case "mr":
        return "e";
      case "bl":
        return "sw";
      case "bc":
        return "s";
      case "br":
        return "se";
      default:
        return "";
    }
  };

  const getTargets = () => {
    if (activeId?.length > 1 && isActive) {
      return [].slice.call(document.querySelectorAll(".target-active"));
    }
    return templateRef;
  };

  const target = getTargets();

  // useEffect(() => {
  //   let totalValue = 0;
  //   targets.map((d) => {
  //     if (d.name === properties.name) {
  //       totalValue++;
  //     }
  //   });
  //   updateElement({
  //     zIndex: targets.length,
  //     index: totalValue,
  //     fontSize: properties?.fontSize,
  //     innerTextValue: `[Collection ${properties?.name} ${totalValue}]`,
  //   });
  // }, [properties?.name]);

  let customStyle = properties?.customCSS;

  const renderElement = (elementName) => {
    let elementStyle = {
      width: width,
      height: height,
      backgroundColor: backgroundColor,
      flexDirection: "column",
      display: "flex",
      justifyContent: "center",
      alignItems: "center",
      borderRadius: borderRadius,
      ...customStyle,
      border: "solid",
      borderColor: strokeColor,
    };

    switch (elementName) {
      case "image":
        return (
          <div style={elementStyle}>
            <img
              src={imagePlaceholder}
              alt="image"
              style={{ width: "100%", height: "100%" }}
            />
            <InputField
              style={{
                position: "absolute",
                textAlign: "center",
                bottom: 0,
                top: 0,
                left: 0,
                right: 0,
                margin: "auto",
                fontSize: (properties?.heightInMm + properties?.widthInMm) / 2,
                background: "transparent",
                border: "none",
                height: properties?.height / 2,
                width: properties?.width,
              }}
              type="text"
              name="imageTextField"
              onChange={(e) =>
                updateElement({
                  innerTextValue: e.target.value,
                })
              }
              value={properties?.innerTextValue}
            />
          </div>
        );

      case "text":
        return (
          <TextField
            isActive={isActive}
            properties={properties}
            updateText={(d) => updateElement({ innerTextValue: d })}
          />
        );

      case "table":
        // return (
        //   <div
        //     style={elementStyle}
        //     dangerouslySetInnerHTML={{ __html: properties?.tableHTML }}
        //   />
        // );
        return parse(properties?.tableHTML);

      case "triangle":
        return (
          <div
            style={{
              width,
              height,
              flexDirection: "column",
              display: "flex",
              justifyContent: "center",
              alignItems: "center",
              borderStyle: "solid",
              borderWidth: `0 ${properties?.width / 2}px ${
                properties?.height
              }px ${properties?.width / 2}px`,
              borderColor: `transparent transparent ${
                customCSS?.backgroundColor || properties?.triangleColor
              } transparent`,
              ...customCSS,
            }}
          ></div>
        );

      default:
        return <div style={elementStyle}></div>;
    }
  };

  const moveToPosition = () => {
    return moveable?.current?.request(
      "draggable",
      {
        x: left,
        y: top,
        isInstant: true,
      },
      true
    );
  };

  const isSet = initialized.includes(properties?.id);

  const getNextSetId = () => {
    const pending = targets.filter((d) => !initialized.includes(d.id));
    if (isValidArray(pending)) {
      return pending[0].id;
    }
    return null;
  };

  const nextSetId = getNextSetId();
  const expansionClassName = detectExpansionFN();

  useEffect(() => {
    let data = [];

    document
      .querySelectorAll(".create-moveable")
      .forEach((element) => data.push(element));

    setElementGuidelines(data);
  }, [isActive]);

  useEffect(() => {
    if (!isSet && nextSetId === properties.id) {
      setTempActive(true);
    }
  }, [isSet, nextSetId, properties.id]);

  useEffect(() => {
    if (!initialized.includes(properties?.id) && tempActive) {
      setInitialized([...initialized, properties?.id]);
      setTimeout(moveToPosition, 0);
      setTimeout(moveToPosition, 10);
      setTimeout(() => {
        setTempActive(false);
      }, 100);
    }
  }, [tempActive, initialized, setInitialized, properties.id]);

  useEffect(() => {
    if (moveable?.current?.style) {
      moveable.current.style.transform = `translate(${left}px, ${top}px) rotate(${rotate}deg)`;
    }
  }, [rotate, left, top]);

  useEffect(() => {
    const moveableLine = document.querySelectorAll(".moveable-control-box")[0];

    if (moveableLine) {
      moveableLine.style.zIndex = selectedElement?.zIndex;
    }
  }, [activeId]);

  if (isLoading) {
    return <Loader />;
  }

  return (
    <React.Fragment>
      <div
        id={properties?.id}
        className={classNames(properties?.classSelector, {
          "create-moveable": !isActive,
          "target-active": isFieldActive,
          [`moveable-${expansionClassName}`]: !!expansionClassName,
        })}
        style={{
          height: height,
          width: width,
          zIndex,
          overflow: "hidden",
          position: "absolute",
          transform: `translate(${left}px, ${top}px) rotate(${rotate}deg)`,
          color: color,
          fontWeight: fontWeight,
          padding: `${padding?.top}px ${padding?.right}px ${padding?.bottom}px ${padding?.left}px`,
        }}
        ref={templateRef}
        onClick={(e) => {
          e.preventDefault();
          e.stopPropagation();
          if (!isActive) {
            if (e?.shiftKey) {
              setActiveId([...activeId, properties?.id]);
            } else {
              if (isValidArray(properties?.groupWith)) {
                setActiveId([...properties.groupWith]);
              } else {
                setActiveId([properties?.id]);
              }
            }
          }
        }}
      >
        {renderElement(properties?.name)}
      </div>

      {!!templateRef?.current && (isActive || tempActive) && (
        <Moveable
          bounds={{
            left: 0,
            top: 0,
            bottom: Number(blockHeight),
            right: Number(blockWidth),
          }}
          ref={moveable}
          target={target}
          {...defaultMoveableProps}
          keepRatio={keepRatio}
          resizable={target.length > 1 ? false : true}
          elementGuidelines={elementGuidelines}
          verticalGuidelines={[0, blockWidth / 2]}
          horizontalGuidelines={[0, blockHeight / 2]}
          onDragStart={helper.onDragStart}
          onResizeStart={(e) => {
            if (e.inputEvent.shiftKey) {
              setKeepRatio(true);
            }
            helper.onResizeStart(e);
          }}
          onDrag={helper.onDrag}
          onResize={helper.onResize}
          onDragEnd={(e) => {
            if (e?.lastEvent) {
              const { translate = [] } = e?.lastEvent || {};

              updateElement({
                left: `${parseFloat(translate?.[0]).toFixed(2) || 0}`,
                top: `${parseFloat(translate?.[1]).toFixed(2) || 0}`,
                leftInMm: `${
                  parseFloat(pxTomm(translate?.[0])).toFixed(2) || 0
                }`,
                topInMm: `${
                  parseFloat(pxTomm(translate?.[1])).toFixed(2) || 0
                }`,
              });
            }
            // }
          }}
          onRotate={helper.onRotate}
          onRotateStart={helper.onRotateStart}
          onRotateEnd={(e) => {
            if (e?.lastEvent) {
              updateElement({
                rotate: Number(e?.lastEvent?.rotate),
              });
            }
          }}
          onResizeEnd={(e) => {
            setKeepRatio(false);
            if (e?.lastEvent) {
              const {
                inputEvent,
                height: updatedHeight = 0,
                width: updatedWidth = 0,
                drag,
              } = e?.lastEvent || {};

              let updated = {
                height: parseInt(`${updatedHeight || 0}`),
                width: parseInt(`${updatedWidth || 0}`),
                widthInMm: parseInt(Number(pxTomm(updatedWidth)) || 0),
                heightInMm: parseInt(Number(pxTomm(updatedHeight)) || 0),
              };

              if (inputEvent?.shiftKey) {
                updated.width = (updated.width * height) / width;
              }

              updateElement({
                width: Number(`${parseFloat(updated.width).toFixed(2) || 0}`),
                height: Number(`${parseFloat(updated.height).toFixed(2) || 0}`),
                widthInMm:
                  Number(parseFloat(pxTomm(updatedWidth)).toFixed(2)) || 0,
                heightInMm:
                  Number(parseFloat(pxTomm(updatedHeight))).toFixed(2) || 0,
                left: `${parseFloat(drag?.translate?.[0]).toFixed(2) || 0}`,
                top: `${parseFloat(drag?.translate?.[1]).toFixed(2) || 0}`,
              });
            }
          }}
          onDragGroup={({ events }) => {
            let updated = [...targets];

            events.forEach((ev) => {
              const { translate = [], target, rotation } = ev || {};
              const id = target.id;

              updated = updated.map((d) => {
                if (d.id === id) {
                  return {
                    ...d,
                    left: parseInt(`${translate?.[0] || 0}`),
                    top: parseInt(`${translate?.[1] || 0}`),
                    id: d.id,
                  };
                }
                return d;
              });
            });

            setTargets(updated);
          }}
          onRotateGroup={({ events }) => {
            let updated = [...targets];

            events.forEach((ev, i) => {
              const { target, rotation, translate = [] } = ev || {};
              const id = target.id;

              updated = updated.map((d) => {
                if (d.id === id) {
                  return {
                    ...d,
                    rotate: parseInt(rotation),
                    left: parseInt(`${ev.drag.beforeTranslate[0] || 0}`),
                    top: parseInt(`${ev.drag.beforeTranslate[1] || 0}`),
                    id: d.id,
                  };
                }
                return d;
              });
            });

            setTargets(updated);
          }}
        />
      )}
    </React.Fragment>
  );
};

export default MoveableElement;
