import React, { useEffect, useRef, useState } from "react";
import { useDispatch, useSelector } from "react-redux";
import uuid from "react-uuid";
import BlockSettingHandler from "./BlockSettingHandler";
import ElementPropertiesHandler from "./ElementPropertiesHandler";
import MoveableElement from "./MoveableElement";
import CanvasHeader from "./CanvasHeader";
import { isEqual } from "lodash";
import { jsPDF } from "jspdf";

import "bootstrap/dist/css/bootstrap.min.css";
import {
  useLocation,
  useParams,
} from "react-router-dom/cjs/react-router-dom.min";
import Loader from "components/Common/Loader";
import { isValidArray, mmTopx } from "modules/utils";
import { fetchGetBlock } from "redux/actions/blockBuilder/getBlock";
import { fetchAllToolbars } from "redux/actions/blockBuilder/getAllToolbars";
import { fetchAddUpdateElement } from "redux/actions/blockBuilder/addUpdateElement";
import { fetchDeleteElement } from "redux/actions/blockBuilder/deleteElement";
import {
  circleTemplate,
  imageTemplate,
  rectangleTemplate,
  tableTemplate,
  textTemplate,
  triangleTemplate,
} from "data/blockBuilder";
import { fetchAddUpdateSingleElement } from "redux/actions/blockBuilder/addUpdateSingleElement";

const NewCanvas = ({ editId }) => {
  console.log("🚀 ~ file: NewCanvas.js:33 ~ NewCanvas ~ editId:", editId);
  const dispatch = useDispatch();

  const layoutID = useLocation().state?.blockId || editId;
  let intervalId = useRef(null);

  const {
    toolbarData = null,
    isToolbarDataLoading = false,
    currentBlock,
  } = useSelector((state) => ({
    toolbarData: state.getAllToolbars?.data?.data,
    isToolbarDataLoading: state.getAllToolbars?.loading,
    currentBlock: state.getBlock?.data?.data,
  }));

  useEffect(() => {
    dispatch(fetchAllToolbars());
  }, []);

  const [blockId, setBlockId] = useState(uuid());
  const [activeId, setActiveId] = useState([]);

  const [attrSetup, setAttrSetup] = useState("");
  const [blockHeight, setBlockHeight] = useState(
    currentBlock?.BlockHeight || 500
  );

  const [blockWidth, setBlockWidth] = useState(currentBlock?.BlockWidth || 500);
  const [displayHeight, setDisplayHeight] = useState();
  const [displayWidth, setDisplayWidth] = useState();
  const [blockSettingUnit, setBlockSettingUnit] = useState("px");
  const [elementPropsUnit, setElementPropsUnit] = useState("px");
  const [blockName, setBlockName] = useState("Test");
  const [targets, setTargets] = useState([]);
  const [isEditable, setIsEditable] = useState(true);
  const [initialized, setInitialized] = useState([]);
  const [templateScale, setTemplateScale] = useState(0.5);
  const [isGroupBTNSelected, setIsGroupBTNSelected] = useState(false);
  const [isTableModelOpen, setIsTableModelOpen] = useState(false);
  const [blockCategory, setBlockCategory] = useState("");

  const handleTableModal = () => {
    setIsTableModelOpen(!isTableModelOpen);
  };

  useEffect(() => {
    getInitialBlockData();
  }, []);

  const { templateScaleDB } = useSelector((state) => ({
    templateScaleDB: state?.getBlock?.data?.data?.BlockScaling,
  }));

  const getInitialBlockData = () => {
    if (layoutID) {
      dispatch(fetchGetBlock(Number(layoutID))).then((d) => {
        setBlockName(d?.data?.BlockName);
        setBlockHeight(
          d?.data?.BlockUnitType === "mm"
            ? mmTopx(parseInt(d?.data?.BlockHeight))
            : d?.data?.BlockHeight
        );
        setBlockWidth(
          d?.data?.BlockUnitType === "mm"
            ? mmTopx(parseInt(d?.data?.BlockWidth))
            : d?.data?.BlockWidth
        );
        setTargets(d?.data?.blocks);
        setBlockSettingUnit(d?.data?.BlockUnitType);
        setElementPropsUnit(d?.data?.BlockUnitType);
        setDisplayHeight(parseInt(d?.data?.BlockHeight));
        setDisplayWidth(parseInt(d?.data?.BlockWidth));
        setTemplateScale(Number(d?.data?.BlockScaling) || 1);
        setBlockCategory(d?.data?.BlockMergeLevel);
      });
    }
  };

  const onBlockSave = () => {
    let formData = {
      blockId: Number(layoutID || blockId),
      canvas: {
        blockId: Number(layoutID || blockId),
        blockName: blockName,
        blockHeight: displayHeight,
        blockWidth: displayWidth,
        blockUnit: blockSettingUnit,
      },
      blocks: [...targets],
    };

    dispatch(fetchAddUpdateElement({ BlockData: formData }));
  };

  const getNewFieldProperties = (type, innerHTML, width) => {
    switch (type) {
      case "circle":
        return circleTemplate(blockHeight, blockWidth);
      case "rectangle":
        return rectangleTemplate(blockHeight, blockWidth);
      case "triangle":
        return triangleTemplate(blockHeight, blockWidth);
      case "table":
        return tableTemplate(innerHTML, parseInt(width));
      case "text":
        return textTemplate(blockHeight, blockWidth);
      case "image":
        return imageTemplate(blockHeight, blockWidth);
      default:
        return {};
    }
  };

  const onAddMore = (currentShape, innerHTML, width, schemaId) => {
    const data = getNewFieldProperties(
      currentShape,
      innerHTML,
      width,
      schemaId
    );
    const id = uuid();
    setTargets([
      ...targets,
      { ...data, id, blockId: blockId, schemaId: parseInt(schemaId) },
    ]);
    setActiveId([id]);
    let formData = {
      blockId: Number(layoutID || blockId),
      canvas: {
        blockId: Number(layoutID || blockId),
        blockName: blockName,
        blockHeight: displayHeight,
        blockWidth: displayWidth,
        blockUnit: blockSettingUnit,
      },
      blocks: [...targets],
    };

    dispatch(fetchAddUpdateElement({ BlockData: formData }));
  };

  const onRemove = () => {
    const afterDelete = targets.filter((d) => !activeId.includes(d.id));

    dispatch(fetchDeleteElement(activeId));
    setTargets([...afterDelete]);
    setActiveId([]);
  };

  const onRemoveAll = () => {
    setTargets([]);
    setActiveId([]);
  };
  let selectedElement = null;
  selectedElement = isValidArray(activeId)
    ? targets.find((d) => activeId.includes(d.id))
    : null;
  selectedElement = { ...selectedElement, blockId: blockId };

  const addUpdateSaveElement = (updated, type, subType = "") => {
    const intervalTime =
      subType === "fontSize" ? 1500 : type === "table" ? 100 : 5000;

    if (intervalId.current) {
      clearInterval(intervalId.current);
    }
    intervalId.current = setInterval(() => {
      if (type === "table") {
        dispatch(fetchAddUpdateSingleElement({ BlockData: updated })).then(
          (data) => {
            if (data) {
              getInitialBlockData();
            }
          }
        );

        if (intervalId.current) {
          clearInterval(intervalId.current);
        }
      } else if (updated !== targets) {
        dispatch(fetchAddUpdateSingleElement({ BlockData: updated }));
        if (intervalId.current) {
          clearInterval(intervalId.current);
        }
      }
    }, intervalTime);
  };

  const updateElement = (data, type, subType) => {
    if (isValidArray(activeId) && activeId.length === 1) {
      const updated = targets.map((d) => {
        const current = activeId.map((mappedData) => d.id === mappedData)[0];
        if (current) {
          addUpdateSaveElement(
            {
              ...d,
              ...data,
              id: d.id,
            },
            type,
            subType
          );
          return {
            ...d,
            ...data,
            id: d.id,
          };
        }
        return d;
      });

      setTargets(updated);
    }
  };

  const updateTemplateScale = (value) =>
    setTemplateScale(
      Number(parseFloat(`${value > 0 ? value : 0.1}`).toFixed(2))
    );

  if (isToolbarDataLoading) {
    return <Loader />;
  }

  const btnDisabled = (btnName) => {
    switch (btnName) {
      case "Bring forward":
        return targets?.length + 1 < selectedElement?.zIndex || !selectedElement
          ? true
          : false;
      case "Send backward":
        return selectedElement?.zIndex < 1 || !selectedElement ? true : false;

      case "Rectangle":
      case "Triangle":
      case "Circle":
      case "Text":
      case "Image":
      case "Table":
      case "Group":
      case "Ungroup":
      case "Layers":
      case "Zoom in":
      case "Zoom out":
        return false;

      case "Bold":
      case "Italic":
      case "Underline":
        return selectedElement?.name === "text" ? false : true;
      case "Delete":
        return !isValidArray(activeId);

      default:
        return !selectedElement ? true : false;
    }
  };

  const onUnGroup = () => {
    if (activeId?.length > 1) {
      const updated = targets.map((d) => {
        if (activeId.includes(d.id)) {
          return {
            ...d,
            groupWith: [],
          };
        }
        return d;
      });

      setTargets(updated);
      setActiveId([]);
    }
  };

  const onGroup = () => {
    if (
      activeId?.length > 1 &&
      !isEqual(activeId, selectedElement?.groupWith || [])
    ) {
      const updated = targets.map((d) => {
        if (activeId.includes(d.id)) {
          return {
            ...d,
            groupWith: activeId,
          };
        }
        return d;
      });

      setTargets(updated);
    }
  };

  const savePdf = () => {
    const doc = new jsPDF();
    doc.text("Producty Pdf!", 10, 10);
    doc.save("a4.pdf");
  };

  const onButtonClick = (btnName) => {
    switch (btnName) {
      case "Rectangle":
      case "Triangle":
      case "Circle":
      case "Text":
      case "Image":
        return onAddMore(btnName.toLowerCase());
      case "Print":
        return savePdf();

      case "Zoom to fit":
        setTemplateScale(Number(templateScaleDB));
        break;

      case "Table":
        setIsTableModelOpen(true);
        break;

      case "Bold":
      case "Italic":
      case "Underline":
        return updateElement({
          [btnName.toLowerCase()]: !selectedElement?.[btnName.toLowerCase()],
        });

      case "Left":
      case "Center":
      case "Right":
        return updateElement({
          textAlign: btnName.toLowerCase(),
        });

      case "Zoom in":
        return updateTemplateScale(templateScale + 0.05);

      case "Zoom out":
        return updateTemplateScale(templateScale - 0.05);

      case "Group":
        return onGroup();

      case "Ungroup":
        onUnGroup();
        break;

      case "Bring forward":
        return updateElement({
          zIndex: selectedElement?.zIndex + 1,
        });
      case "Send backward":
        return updateElement({
          zIndex: selectedElement?.zIndex - 1,
        });
      case "Bring to front":
        return updateElement({
          zIndex: targets.length + 1,
        });
      case "Send to back":
        return updateElement({
          zIndex: 0,
        });

      case "Save":
        return onBlockSave();

      case "Rotate":
        return updateElement({
          rotate: selectedElement?.rotate + 180,
        });
      case "Rotate 90&deg; clockwise":
        return updateElement({
          rotate: selectedElement?.rotate + 90,
        });
      case "Rotate 90&deg; counter-clockwise":
        return updateElement({
          rotate: selectedElement?.rotate - 90,
        });
      case "Delete":
        return onRemove();
      default:
        break;
    }
  };

  // setTemplateScaleInitial=()

  const renderToolbarIcons = () => {
    const filterData = toolbarData
      ? toolbarData.filter((d) => {
          return d?.ToolbarID !== 5 || d?.ToolbarName !== "Draw";
        })
      : [];

    return filterData?.map((d, i) => {
      return (
        <div className="my-1" key={i}>
          {d.iconGroups.map((d, i) => {
            return (
              <span key={i} className="btns-wrapper">
                {d.icons.map((d, i) => (
                  <button
                    key={i}
                    className={
                      d.ButtonName === "Group" && isGroupBTNSelected
                        ? "btn selected"
                        : "btn"
                    }
                    disabled={btnDisabled(d.ButtonName)}
                    onClick={() => onButtonClick(d.ButtonName)}
                  >
                    <i className={d?.ButtonIcon} title={d?.ButtonName} />
                  </button>
                ))}
              </span>
            );
          })}
        </div>
      );
    });
  };

  return (
    <>
      <div className="shape-btns" id="nav-toolbar">
        {renderToolbarIcons()}
      </div>
      <div className="block-builder-section d-flex">
        <BlockSettingHandler
          attrSetup={attrSetup}
          blockHeight={blockHeight}
          blockSettingUnit={blockSettingUnit}
          blockName={blockName}
          blockWidth={blockWidth}
          onAddMore={onAddMore}
          onRemove={onRemove}
          onRemoveAll={onRemoveAll}
          setAttrSetup={setAttrSetup}
          setBlockHeight={setBlockHeight}
          setBlockSettingUnit={setBlockSettingUnit}
          setBlockName={setBlockName}
          setBlockWidth={setBlockWidth}
          targets={targets}
          activeId={activeId}
          elementPropsUnit={elementPropsUnit}
          setElementPropsUnit={setElementPropsUnit}
          updateElement={updateElement}
          setTargets={setTargets}
          selectedElement={selectedElement}
          displayHeight={displayHeight}
          displayWidth={displayWidth}
          privateProps={selectedElement}
          toolbarData={toolbarData}
          setIsTableModelOpen={handleTableModal}
          isTableModelOpen={isTableModelOpen}
          blockId={layoutID || blockId}
        />
        <div className="main-template position-relative">
          <div className="main-header-template">
            <CanvasHeader
              onAddMore={onAddMore}
              onRemove={onRemove}
              selectedElement={selectedElement}
              onRemoveAll={onRemoveAll}
              targets={targets}
              setTargets={setTargets}
              updateElement={updateElement}
              toolbarData={toolbarData}
              templateScale={templateScale}
              // setTemplateScale={setTemplateScaleInitial}
              updateTemplateScale={updateTemplateScale}
              activeId={activeId}
              setActiveId={setActiveId}
              isGroupBTNSelected={isGroupBTNSelected}
              setIsGroupBTNSelected={setIsGroupBTNSelected}
              onBlockSave={onBlockSave}
              blockWidth={blockWidth}
            />
          </div>
          {layoutID && (
            <div className="template-content-outer w-100">
              <div
                className={`template-content ${
                  templateScale > templateScaleDB
                    ? "overflow-auto"
                    : "overflow-hidden"
                }`}
                style={{
                  width: "100% !important",
                  minWidth: "100% !important",
                  maxWidth: "100% !important",
                }}
              >
                <div
                  className="template-content-inner"
                  onClick={() => {
                    setActiveId([]);
                  }}
                >
                  <div
                    className=""
                    style={{
                      // height: "100%",
                      minHeight: Number(blockHeight) * templateScale,
                      minWidth: Number(blockWidth) * templateScale,
                      height: "100%",
                      width: "100%",
                      display: "flex",
                      justifyContent: "center",
                      alignItems: "center",
                    }}
                  >
                    <div
                      className=""
                      id="main-template-content"
                      style={{
                        // transform: `scale(${templateScale})`,
                        // position: "relative",
                        // backgroundColor: "gray",
                        // height: Number(blockHeight),
                        // minHeight: Number(blockHeight),
                        // maxHeight: Number(blockHeight),
                        // width: Number(blockWidth),
                        // minWidth: Number(blockWidth),
                        // maxWidth: Number(blockWidth),

                        height: Number(blockHeight),
                        width: Number(blockWidth),
                        minHeight: Number(blockHeight),
                        minWidth: Number(blockWidth),
                        maxHeight: Number(blockHeight),
                        maxWidth: Number(blockWidth),
                        transform: `scale(${templateScale})`,
                        zIndex: 2,
                        background: "white",
                        transformOrigin: `top`,
                      }}
                      onClick={(e) => {
                        e.preventDefault();
                        e.stopPropagation();
                        setActiveId([]);
                      }}
                    >
                      {targets?.map((target) => (
                        <MoveableElement
                          activeId={activeId}
                          blockHeight={blockHeight}
                          blockSettingUnit={blockSettingUnit}
                          blockWidth={blockWidth}
                          elementPropsUnit={elementPropsUnit}
                          initialized={initialized}
                          isActive={activeId.includes(target.id)}
                          isEditable={isEditable}
                          key={target?.id}
                          properties={target}
                          setActiveId={setActiveId}
                          setBlockSettingUnit={setBlockSettingUnit}
                          setInitialized={setInitialized}
                          setIsEditable={setIsEditable}
                          setTargets={setTargets}
                          targets={targets}
                          updateElement={updateElement}
                          selectedElement={selectedElement}
                          templateScale={templateScale}
                          setIsGroupBTNSelected={setIsGroupBTNSelected}
                          isGroupBTNSelected={isGroupBTNSelected}
                        />
                      ))}
                    </div>
                  </div>
                </div>
              </div>
            </div>
          )}

          {/* <div className="w-100">
          <div
            className="shadow"
            style={{
              overflow: "auto",
              minHeight: Number(blockHeight) * templateScale,
              minWidth: Number(blockWidth) * templateScale,
            }}
          >
            <div
              style={{
                transform: `scale(${templateScale})`,
                position: "relative",
                backgroundColor: "gray",
                height: Number(blockHeight),
                width: Number(blockWidth),
              }}
              className="shadow-inner"
              onClick={(e) => {
                e.preventDefault();
                e.stopPropagation();
                setActiveId([]);
              }}
            >
              {targets?.map((target) => (
                <MoveableElement
                  activeId={activeId}
                  blockHeight={blockHeight}
                  blockSettingUnit={blockSettingUnit}
                  blockWidth={blockWidth}
                  elementPropsUnit={elementPropsUnit}
                  initialized={initialized}
                  isActive={activeId.includes(target.id)}
                  isEditable={isEditable}
                  key={target?.id}
                  properties={target}
                  setActiveId={setActiveId}
                  setBlockSettingUnit={setBlockSettingUnit}
                  setInitialized={setInitialized}
                  setIsEditable={setIsEditable}
                  setTargets={setTargets}
                  targets={targets}
                  updateElement={updateElement}
                  selectedElement={selectedElement}
                  templateScale={templateScale}
                />
              ))}
            </div>
          </div>
        </div> */}
        </div>
        {/* <ElementPropertiesHandler
          activeId={activeId}
          isEditable={isEditable}
          privateProps={selectedElement}
          setIsEditable={setIsEditable}
          updateElement={updateElement}
          targets={targets}
          blockHeight={blockHeight}
          blockWidth={blockWidth}
          blockSettingUnit={blockSettingUnit}
          elementPropsUnit={elementPropsUnit}
          setElementPropsUnit={setElementPropsUnit}
        /> */}
        <ElementPropertiesHandler
          activeId={activeId}
          attrSetup={attrSetup}
          blockHeight={currentBlock?.BlockHeight}
          blockId={layoutID || blockId}
          blockName={blockName}
          blockSettingUnit={blockSettingUnit}
          blockWidth={currentBlock?.BlockWidth}
          displayHeight={displayHeight}
          displayWidth={displayWidth}
          elementPropsUnit={elementPropsUnit}
          isEditable={isEditable}
          onAddMore={onAddMore}
          onRemove={onRemove}
          onRemoveAll={onRemoveAll}
          privateProps={selectedElement}
          selectedElement={selectedElement}
          setActiveId={setActiveId}
          setAttrSetup={setAttrSetup}
          setBlockHeight={setBlockHeight}
          setBlockName={setBlockName}
          setBlockSettingUnit={setBlockSettingUnit}
          setBlockWidth={setBlockWidth}
          setElementPropsUnit={setElementPropsUnit}
          setIsEditable={setIsEditable}
          setTargets={setTargets}
          targets={targets}
          updateElement={updateElement}
          updateElementChanges={addUpdateSaveElement}
          blockCategory={blockCategory}
        />
      </div>
    </>
  );
};

export default NewCanvas;
