import React from "react";
import * as Yup from "yup";

const errorMessages = {
  required: "Required",
  invalidValue: "Value is invalid",
  invalidEmail: "Invalid email",
  numberNotZero: "Value has to be bigger than 0,01",
};

export const isEmail = (value) => /^[^\s@]+@[^\s@]+\.[^\s@]+$/.test(value);
export const isNotBadWord = (value) => /^[\\(INST,\\[,]+$/.test(value);

export const isAlphabetic = (value) => /^[a-zA-Z]+$/.test(value);

export const numberTransform = (value) =>
  !Number.isNaN(value) ? Number(value) : 0;

export const getErrorMessage = (key) => errorMessages?.[key] ?? key;
export const getCustomErrorMessage = (fieldName) =>
  fieldName != "" ? `The ${fieldName} field is required.` : "Required";

export const validator = (type, fieldName = "") => {
  switch (type) {
    case "onlyString":
      return Yup.string()
        .required(getErrorMessage("required"))
        .test("isAlphabetic", getErrorMessage("invalidValue"), (value) =>
          value !== undefined ? isAlphabetic(value) : true
        );

    case "notRequired":
      return Yup.string().notRequired();

    case "notRequiredMixed":
      return Yup.mixed().notRequired();

    case "notRequiredNumber":
      return Yup.number().transform(numberTransform).notRequired();

    case "notRequiredDate":
      return Yup.date().notRequired();

    case "requiredDate":
      return Yup.date().required(getErrorMessage("required"));

    case "required":
      return Yup.string().required(getErrorMessage("required"));

    case "requiredField":
      return Yup.string().required(getCustomErrorMessage(fieldName));

    case "numberRequired":
      return Yup.number()
        .transform(numberTransform)
        .typeError(getErrorMessage("invalidValue"))
        .required(getErrorMessage("required"));

    case "priceRequired":
      return Yup.number()
        .transform(numberTransform)
        .typeError(getErrorMessage("invalidValue"))
        .required(getErrorMessage("required"))
        .min(1, getErrorMessage("invalidValue"));

    case "email":
      return Yup.string()
        .required(getErrorMessage("required"))
        .email(getErrorMessage("invalidEmail"));

    default:
      return Yup.string().required(getErrorMessage("required"));
  }
};

export const loginValidation = () =>
  Yup.object().shape({
    userName: validator("email"),
    password: validator("required"),
  });

export const displayFormErrors = (
  key = "",
  errors = {},
  touched = {},
  submitCount = 1,
  fieldName = ""
) => {
  if (errors[key] !== undefined && errors[key] && submitCount) {
    return (
      <div className="text-danger input-feedback font12">{errors[key]}</div>
    );
  }
  return null;
};

export const addOrganizationForm = () =>
  Yup.object().shape({
    organizationName: validator("required"),
  });

export const addCatalogForm = () =>
  Yup.object().shape({
    catalogName: validator("required"),
    catalogOnSaleDate: validator("required").nullable(),
  });

export const inviteSellerForm = () =>
  Yup.object().shape({
    email: validator("email"),
  });

export const inviteAccountManagerForm = () =>
  Yup.object().shape({
    email: validator("email"),
  });

export const inviteUserForm = () =>
  Yup.object().shape({
    email: validator("email"),
  });

export const addProductForm = () =>
  Yup.object().shape({
    searchProductCode: validator("required"),
  });

export const addImageLabelForm = () =>
  Yup.object().shape({
    label: validator("required"),
  });

export const addImageSizeForm = () =>
  Yup.object().shape({
    width: validator("required"),
    height: validator("required"),
  });

export const unitForm = () =>
  Yup.object().shape({
    UnitAbbreviation: validator("required"),
    UnitDescription: validator("required"),
  });

export const textBlockForm = () =>
  Yup.object().shape({
    textBlockName: validator("required"),
  });

export const addTemplateForm = () =>
  Yup.object().shape({
    templateName: validator("required"),
  });

export const addAttributeForm = () =>
  Yup.object().shape({
    AttributeName: validator("required"),
  });

export const editProductBlobForm = () => Yup.object().shape({});

export const addProductBlobForm = () =>
  Yup.object().shape({
    blobName: validator("requiredField", "Blob name"),
    addBlobData: validator("requiredField", "Blob data"),
  });

export const languageForm = () =>
  Yup.object().shape({
    language: Yup.object().required(),
  });
export const attributeSchemaForm = () =>
  Yup.object().shape({
    LayoutName: validator("required"),
  });

export const catalogSetupSchemaForm = () =>
  Yup.object().shape({
    label: validator("required"),
  });

export const attributeSchemaListForm = () =>
  Yup.object().shape({
    label: validator("required"),
  });

export const textBlockSchemaListForm = () =>
  Yup.object().shape({
    label: validator("required"),
  });

export const wpIntegrationForm = () =>
  Yup.object().shape({
    storeUrl: validator("requiredField", "store url"),
    consumerKey: validator("requiredField", "consumer key"),
    secretKey: validator("requiredField", "secret key"),
  });

export const resetPasswordForm = () =>
  Yup.object().shape({
    email: validator("email"),
  });

export const changePasswordForm = () =>
  Yup.object().shape({
    password: validator("required"),
    confirmPassword: Yup.string().oneOf(
      [Yup.ref("password"), null],
      "Passwords must match"
    ),
  });

export const registerValidation = () =>
  Yup.object().shape({
    firstName: Yup.string().required(getErrorMessage("First name is required")),
    lastName: Yup.string().required(getErrorMessage("Last name is required")),
    // phone: Yup.string().required(getErrorMessage("Phone number is required")),
    email: Yup.string()
      .required(getErrorMessage("Email is required"))
      .email(getErrorMessage("Please enter a valid email address")),
    organisationName: Yup.string().required(
      getErrorMessage("Organisation name is required")
    ),
    // organisationType: Yup.string().required(
    //   getErrorMessage("Organisation type is required")
    // ),
    password: Yup.string().required(getErrorMessage("Password is required")),
    confirmPassword: Yup.string()
      .required("Confirm password is required")
      .oneOf([Yup.ref("password"), null], "Passwords must match"),
  });
